# -*- coding: utf-8 -*-
from django.db import models
from django import forms

from .widgets import EpicEditorWidget

from markdown import markdown


class EpicEditorField(models.TextField):

    def __init__(self, *args, **kwargs):
        self._markdown_safe = not kwargs.pop('allow_html', True)
        self._html_field_suffix = kwargs.pop('html_field_suffix', '_html')
        self.configs = kwargs.pop("configs", None)
        super(EpicEditorField, self).__init__(*args, **kwargs)

    def contribute_to_class(self, cls, name):
        self._html_field = "%s%s" % (name, self._html_field_suffix)
        models.TextField(editable=False).contribute_to_class(
            cls, self._html_field)
        super(EpicEditorField, self).contribute_to_class(cls, name)

    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.attname)
        html = markdown(value, safe_mode=self._markdown_safe)
        setattr(model_instance, self._html_field, html)
        return value

    def formfield(self, **kwargs):
        defaults = {
            'form_class': EpicEditorFormField,
            'configs': self.configs,
        }
        defaults.update(kwargs)
        return super(EpicEditorField, self).formfield(**defaults)

    def __unicode__(self):
        return self.attname


class EpicEditorFormField(forms.fields.Field):

    def __init__(self, configs=None, *args, **kwargs):
        kwargs.update({'widget': EpicEditorWidget(configs=configs)})
        super(EpicEditorFormField, self).__init__(*args, **kwargs)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^epiced\.fields\.EpicEditorField"])
except:
    pass



from django import forms
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
from django.utils.encoding import force_unicode
from django.core.exceptions import ImproperlyConfigured
from django.forms.util import flatatt
import json

DEFAULT_EPICEDITOR_CONFIG = {
    "container": 'epiceditor',
    "textarea": None,
    "basePath": settings.STATIC_URL + 'epiced',
    "clientSideStorage": False,
    "localStorageName": 'epiceditor',
    "useNativeFullscreen": True,
    "parser": 'marked',
    "file": {
        "name": 'epiceditor',
        "defaultContent": '',
        "autoSave": 100
    },
    "theme": {
        "base": '/themes/base/epiceditor.css',
        "preview": '/themes/preview/preview-dark.css',
        "editor": '/themes/editor/epic-dark.css'
    },
    "button": {
        "preview": True,
        "fullscreen": True
    },
    "focusOnLoad": True,
    "shortcut": {
        "modifier": 18,
        "fullscreen": 70,
        "preview": 80
    },
    "string": {
        "togglePreview": 'Toggle Preview Mode',
        "toggleEdit": 'Toggle Edit Mode',
        "toggleFullscreen": 'Enter Fullscreen'
    }
}


class EpicEditorWidget(forms.Textarea):

    """
    Widget providing EpicEditor for Markdown Editing.
    """
    class Media:
        try:
            if settings.DEBUG is True:
                js = (
                    settings.STATIC_URL + 'epiced/js/epiceditor.js',
                )
            else:
                js = (
                    settings.STATIC_URL + 'epiced/js/epiceditor.min.js',
                )
        except AttributeError:
            raise ImproperlyConfigured("django-epiced requires \
                    STATIC_URL setting. This setting specifies a \
                    URL prefix to the epiceditor JS and CSS Statics")

    def __init__(self, configs=None, *args, **kwargs):
        super(EpicEditorWidget, self).__init__(*args, **kwargs)
        # Setup config from defaults.
        self.config = DEFAULT_EPICEDITOR_CONFIG.copy()

        # Try to get valid config from settings.
        settings_configs = getattr(settings, 'EPICEDITOR_CONFIGS', None)
        if settings_configs:
            self.config.update(settings_configs)
        if configs:
            self.config.update(configs)

    def render(self, name, value, attrs={}):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        self.config['container'] = "%s_epiceditor" % (final_attrs['id'], )
        self.config['textarea'] = final_attrs['id']
        return mark_safe(render_to_string('epiced/widget.html', {
            'final_attrs': flatatt(final_attrs),
            'parser': self.config['parser'],
            'field_id': final_attrs['id'],
            'value': conditional_escape(force_unicode(value)),
            'config': json.JSONEncoder().encode(self.config)
        }))
